# -*- coding: utf-8 -*-

from contextlib import contextmanager

from .proxy import WebDriverProxy


DEFAULT_WAIT_TIMEOUT = None
DEFAULT_POLLING_TIMEOUT = 10
DEFAULT_POLLING_DELAY = None
DEFAULT_LOAD_PAGE_TIMEOUT = 10
DEFAULT_SCRIPT_TIMEOUT = None
DEFAULT_MAXIMIZE_WINDOW = True
DEFAULT_WINDOW_SIZE = None


def create(selenium, driver):
    browser = WebDriverProxy(
        driver,
        config=BrowserConfig(selenium, driver),
    )
    browser.reason_storage['browser name'] = selenium.browser_name
    return browser


@contextmanager
def change_config(browser,
                  wait_timeout=None,
                  polling_delay=None,
                  polling_timeout=None,
                  page_load_timeout=None):
    to_restore = {}

    if wait_timeout:
        to_restore['wait_timeout'] = browser.config.WAIT_TIMEOUT
        browser.config.WAIT_TIMEOUT = wait_timeout

    if polling_delay:
        to_restore['polling_delay'] = browser.config.POLLING_DELAY
        browser.config.POLLING_DELAY = polling_delay

    if polling_timeout:
        to_restore['polling_timeout'] = browser.config.POLLING_TIMEOUT
        browser.config.POLLING_TIMEOUT = polling_timeout

    if page_load_timeout:
        to_restore['page_load_timeout'] = browser.config.PAGE_LOAD_TIMEOUT
        browser.config.PAGE_LOAD_TIMEOUT = page_load_timeout

    try:
        yield
    finally:
        if 'wait_timeout' in to_restore:
            browser.config.WAIT_TIMEOUT = to_restore['wait_timeout']

        if 'polling_delay' in to_restore:
            browser.config.POLLING_DELAY = to_restore['polling_delay']

        if 'polling_timeout' in to_restore:
            browser.config.POLLING_TIMEOUT = to_restore['polling_timeout']

        if 'page_load_timeout' in to_restore:
            browser.config.PAGE_LOAD_TIMEOUT = to_restore['page_load_timeout']


class BrowserConfig(object):

    def __init__(self, selenium, driver):
        self.__driver = driver

        self.PROJECT_URL = selenium.config.get('PROJECT_URL')

        self.POLLING_DELAY = selenium.config.get(
            'POLLING_DELAY', DEFAULT_POLLING_DELAY,
        )
        self.POLLING_TIMEOUT = selenium.config.get(
            'POLLING_TIMEOUT', DEFAULT_POLLING_TIMEOUT,
        )

        self.__PAGE_LOAD_TIMEOUT = None
        self.PAGE_LOAD_TIMEOUT = selenium.config.get(
            'PAGE_LOAD_TIMEOUT', DEFAULT_LOAD_PAGE_TIMEOUT,
        )

        self.__SCRIPT_TIMEOUT = None
        self.SCRIPT_TIMEOUT = selenium.config.get(
            'SCRIPT_TIMEOUT', DEFAULT_SCRIPT_TIMEOUT,
        )

        self.__WAIT_TIMEOUT = None
        self.WAIT_TIMEOUT = selenium.config.get(
            'WAIT_TIMEOUT', DEFAULT_WAIT_TIMEOUT,
        )

        self.__WINDOW_SIZE = None
        self.WINDOW_SIZE = selenium.config.get(
            'WINDOW_SIZE', DEFAULT_WINDOW_SIZE,
        )

        self.__MAXIMIZE_WINDOW = False
        self.MAXIMIZE_WINDOW = selenium.config.get(
            'MAXIMIZE_WINDOW', DEFAULT_MAXIMIZE_WINDOW,
        )

    @property
    def PAGE_LOAD_TIMEOUT(self):
        return self.__PAGE_LOAD_TIMEOUT

    @PAGE_LOAD_TIMEOUT.setter
    def PAGE_LOAD_TIMEOUT(self, value):
        if value is not None:
            self.__PAGE_LOAD_TIMEOUT = value
            self.__driver.set_page_load_timeout(value)

    @property
    def SCRIPT_TIMEOUT(self):
        return self.__SCRIPT_TIMEOUT

    @SCRIPT_TIMEOUT.setter
    def SCRIPT_TIMEOUT(self, value):
        if value is not None:
            self.__SCRIPT_TIMEOUT = value
            self.__driver.set_script_timeout(value)

    @property
    def WAIT_TIMEOUT(self):
        return self.__WAIT_TIMEOUT

    @WAIT_TIMEOUT.setter
    def WAIT_TIMEOUT(self, value):
        if value is not None:
            self.__WAIT_TIMEOUT = value
            self.__driver.implicitly_wait(value)

    @property
    def WINDOW_SIZE(self):
        return self.__WINDOW_SIZE

    @WINDOW_SIZE.setter
    def WINDOW_SIZE(self, value):
        if value:
            self.__WINDOW_SIZE = value
            self.__driver.set_window_size(
                *self.WINDOW_SIZE
            )

    @property
    def MAXIMIZE_WINDOW(self):
        return self.__MAXIMIZE_WINDOW

    @MAXIMIZE_WINDOW.setter
    def MAXIMIZE_WINDOW(self, value):
        if value and not self.__WINDOW_SIZE:
            self.__MAXIMIZE_WINDOW = value
            self.__driver.maximize_window()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from gevent.pool import Pool

from .. import runnable
from ..groups import get_pool_size_of_value
from ..exceptions import ALLOW_RAISED_EXCEPTIONS


def target(runnable_object, result):
    runnable_object(result)


class GeventSuiteGroup(runnable.RunnableGroup):

    def __run__(self, result):
        self._is_run = True

        pool = Pool(
            get_pool_size_of_value(
                self.config.ASYNC_SUITES,
            ),
        )

        try:
            for suite in self.objects:
                pool.spawn(target, suite, result)

            pool.join()
        except ALLOW_RAISED_EXCEPTIONS:
            pool.kill()
            raise


class GeventCaseGroup(runnable.RunnableGroup):

    def __run__(self, result):
        self._is_run = True

        pool = Pool(
            get_pool_size_of_value(
                self.config.ASYNC_TESTS, in_two=True,
            ),
        )

        try:
            for case in self.objects:
                pool.spawn(target, case, result)

            pool.join()
        except ALLOW_RAISED_EXCEPTIONS:
            pool.kill()
            raise



# -*- coding: utf-8 -*-

"""
Module for configuration global context
"""

import seismograph.case as _case
import seismograph.xunit as _xunit
import seismograph.suite as _suite
import seismograph.loader as _loader
import seismograph.program as _program
import seismograph.runnable as _runnable


shared_data = _program.Program.shared_data
shared_extension = _program.Program.shared_extension


def match_suite_to_layer(cls, layer):
    _suite.MATCH_SUITE_TO_LAYER[cls] = layer


def match_case_to_layer(cls, layer):
    _case.MATCH_CASE_TO_LAYER[cls] = layer


def set_default_case_layers(*layers):
    _case.DEFAULT_LAYERS.extend(layers)


def set_default_suite_layers(*layers):
    _suite.DEFAULT_LAYERS.extend(layers)


def set_default_program_layers(*layers):
    _program.DEFAULT_LAYERS.extend(layers)


def configure(
        round_runtime=None,
        config_env_name=None,
        test_name_prefix=None,
        case_group_class=None,
        suite_group_class=None,
        default_test_name=None,
        skip_attribute_name=None,
        skip_why_attribute_name=None,
        use_static_test_functions=False):
    """
    Configure global context

    :param round_runtime:
    :param config_env_name:
    :param test_name_prefix:
    :param case_group_class:
    :param suite_group_class:
    :param default_test_name:
    :param skip_attribute_name:
    :param skip_why_attribute_name:
    :param use_static_test_functions:
    """
    if round_runtime:
        assert type(round_runtime) == int
        _xunit.ROUND_RUNTIME = round_runtime

    if use_static_test_functions:
        _case.Case.__static__ = True

    if config_env_name:
        _program.CONFIG_ENV_NAME = config_env_name

    if test_name_prefix:
        _loader.TEST_NAME_PREFIX = test_name_prefix

    if default_test_name:
        _loader.DEFAULT_TEST_NAME = default_test_name

    if skip_attribute_name:
        _case.SKIP_ATTRIBUTE_NAME = skip_attribute_name

    if skip_why_attribute_name:
        _case.SKIP_WHY_ATTRIBUTE_NAME = skip_why_attribute_name

    if case_group_class:
        assert issubclass(case_group_class, _runnable.RunnableGroup)
        _suite.Suite.__case_group_class__ = case_group_class

    if suite_group_class:
        assert issubclass(suite_group_class, _runnable.RunnableGroup)
        _program.Program.__suite_group_class__ = suite_group_class



# -*- coding: utf-8 -*-


class SeismographError(BaseException):

    def __init__(self, message, *args, **kwargs):
        super(SeismographError, self).__init__(message, *args, **kwargs)

        self.message = message  # please python 3


class Skip(SeismographError):
    pass


class LoaderError(SeismographError):
    pass


class ConfigError(SeismographError):
    pass


class EmergencyStop(SeismographError):
    pass


class CollectError(SeismographError):
    pass


class PyVersionError(SeismographError):
    pass


class TimeoutException(SeismographError):
    pass


class ExtensionNotFound(SeismographError):
    pass


class ExtensionNotRequired(SeismographError):
    pass


ALLOW_RAISED_EXCEPTIONS = (
    EmergencyStop,
    KeyboardInterrupt,
)



# -*- coding: utf-8 -*-

import json
import pickle
import marshal

from .utils import pyv


XML_VERSION = '1.0'
XML_ENCODING = 'UTF-8'

ROUND_RUNTIME = 3


class XUnitData(object):

    def __init__(self,
                 exc=None,
                 reason=None,
                 runtime=None,
                 exc_type=None,
                 class_name=None,
                 method_name=None,
                 exc_message=None):
        if exc:
            self.parse_exc(exc)
        else:
            self.__exc_type = exc_type
            self.__exc_message = exc_message

        self.__reason = reason
        self.__runtime = runtime
        self.__class_name = class_name
        self.__method_name = method_name

    @classmethod
    def from_dict(cls, dct):
        return cls(**dct)

    @classmethod
    def from_json(cls, string):
        return cls(**json.loads(string))

    @classmethod
    def from_pickle(cls, string):
        return cls(**pickle.loads(string))

    @classmethod
    def from_marshal(cls, string):
        return cls(**marshal.loads(string))

    @property
    def reason(self):
        return self.__reason

    @reason.setter
    def reason(self, value):
        self.__reason = value

    @property
    def runtime(self):
        return round(self.__runtime, ROUND_RUNTIME)

    @property
    def exc_type(self):
        return self.__exc_type

    @property
    def class_name(self):
        return self.__class_name

    @property
    def exc_message(self):
        return self.__exc_message or 'Exception was raised without message. Look at reason.'

    @exc_message.setter
    def exc_message(self, value):
        self.__exc_message = value

    @property
    def method_name(self):
        return self.__method_name

    def to_dict(self):
        return {
            'reason': self.__reason,
            'runtime': self.__runtime,
            'exc_type': self.__exc_type,
            'class_name': self.__class_name,
            'exc_message': self.__exc_message,
            'method_name': self.__method_name,
        }

    def parse_exc(self, exc):
        self.__exc_type = '{}.{}'.format(
            exc.__class__.__module__, exc.__class__.__name__,
        )
        self.__exc_message = pyv.get_exc_message(exc)

    def to_json(self):
        return json.dumps(self.to_dict())

    def to_pickle(self):
        return pickle.dumps(self.to_dict())

    def to_marshal(self):
        return marshal.dumps(self.to_dict())


def dict_to_tag_attributes(dct):
    string = u' ' + u' '.join(
        (
            u'{}="{}"'.format(k, screening_line(str(v)))
            for k, v in dct.items() if v is not None
        )
    )

    if string == ' ':
        return ''

    return string


def screening_line(string):
    string = string.replace('<', '&lt;')
    string = string.replace('>', '&gt;')
    string = string.replace('"', '&quot;')
    return string


def cdata(string):
    return u'<![CDATA[{}]]>'.format(string)


def to_xml_tag(tag_name, contains, **attributes):
    if contains:
        return u'<{0}{1}>{2}</{0}>'.format(
            tag_name, dict_to_tag_attributes(attributes), contains,
        )

    return u'<{}{} />'.format(
        tag_name, dict_to_tag_attributes(attributes))


def create_xml_document(result):
    def render_result_proxy(result_proxy):
        cases_report = []

        for _, xunit_data in result_proxy.successes:
            cases_report.append(
                to_xml_tag('testcase', None,
                           time=xunit_data.runtime,
                           name=xunit_data.method_name,
                           classname=xunit_data.class_name,
                           ),
            )

        for _, xunit_data in result_proxy.skipped:
            cases_report.append(
                to_xml_tag('testcase',
                           to_xml_tag('skipped',
                                      cdata(xunit_data.reason),
                                      ),
                           time=xunit_data.runtime,
                           name=xunit_data.method_name,
                           classname=xunit_data.class_name,
                           ),
            )

        for _, xunit_data in result_proxy.failures:
            cases_report.append(
                to_xml_tag('testcase',
                           to_xml_tag('failure',
                                      cdata(xunit_data.reason),
                                      type=xunit_data.exc_type,
                                      message=xunit_data.exc_message,
                                      ),
                           time=xunit_data.runtime,
                           name=xunit_data.method_name,
                           classname=xunit_data.class_name,
                           ),
            )

        for _, xunit_data in result_proxy.errors:
            cases_report.append(
                to_xml_tag('testcase',
                           to_xml_tag('error',
                                      cdata(xunit_data.reason),
                                      type=xunit_data.exc_type,
                                      message=xunit_data.exc_message,
                                      ),
                           time=xunit_data.runtime,
                           name=xunit_data.method_name,
                           classname=xunit_data.class_name,
                           ),
            )

        state = result_proxy.get_state()

        return to_xml_tag('testsuite',
                          u''.join(cases_report),
                          name=result_proxy.name,
                          tests=state.tests,
                          time=state.runtime,
                          skip=state.skipped,
                          errors=state.errors,
                          failures=state.failures,
                          )

    data = u''.join(
        (
            u'<?xml version="{version}" encoding="{encoding}"?>'.format(
                version=XML_VERSION,
                encoding=XML_ENCODING,
            ),
            to_xml_tag('testsuites',
                       u''.join(
                           map(render_result_proxy, result.proxies),
                       )
                       if result.proxies else render_result_proxy(result),
                       name=result.name,
                       tests=result.current_state.tests,
                       time=result.current_state.runtime,
                       skip=result.current_state.skipped,
                       errors=result.current_state.errors,
                       failures=result.current_state.failures,
                       ),
        ),
    )

    if pyv.IS_PYTHON_2:
        return data.encode('utf-8')

    return data



# -*- coding: utf-8 -*-

import logging

from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

from . import drivers
from . import browser
from .polling import POLLING_EXCEPTIONS
from ...utils.common import waiting_for
from .exceptions import SeleniumExError


logger = logging.getLogger(__name__)


EX_NAME = 'selenium'
DEFAULT_GET_BROWSER_DELAY = 0.5
DEFAULT_GET_BROWSER_TIMEOUT = 10
DEFAULT_BROWSER = drivers.FIREFOX

DRIVER_TO_CAPABILITIES = {
    drivers.OPERA: DesiredCapabilities.OPERA,
    drivers.CHROME: DesiredCapabilities.CHROME,
    drivers.FIREFOX: DesiredCapabilities.FIREFOX,
    drivers.PHANTOMJS: DesiredCapabilities.PHANTOMJS,
    drivers.IE: DesiredCapabilities.INTERNETEXPLORER,
}


def get_capabilities(driver_name):
    """
    Get capabilities of driver

    :param driver_name: driver name
    :type driver_name: str
    """
    try:
        return DRIVER_TO_CAPABILITIES[driver_name]
    except KeyError:
        raise SeleniumExError(
            'Capabilities for driver "{}" is not found'.find(driver_name),
        )


class Selenium(object):

    def __init__(self, config):
        self.__config = config

        self.__browser = None
        self.__is_running = False
        self.__browser_name = None

    def __enter__(self):
        self.start()
        return self.__browser

    def __exit__(self, *args, **kwargs):
        self.stop()

    @property
    def config(self):
        return self.__config

    @property
    def browser(self):
        return self.__browser

    @property
    def browser_name(self):
        return self.__browser_name

    @property
    def is_running(self):
        return self.__is_running

    def start(self, browser_name=None, timeout=None, delay=None):
        if not self.__is_running:
            self.__browser_name = (
                browser_name or self.__config.get('DEFAULT_BROWSER', DEFAULT_BROWSER)
            ).lower()

            def get_browser(func, *args, **kwargs):
                try:
                    return func(*args, **kwargs)
                except POLLING_EXCEPTIONS:
                    return None

            def get_local_browser(browser_name):
                method = getattr(self, browser_name, None)

                if method:
                    return method()

                raise SeleniumExError(
                    'Incorrect browser name "{}"'.format(browser_name),
                )

            delay = delay or self.__config.get(
                'POLLING_DELAY', DEFAULT_GET_BROWSER_DELAY,
            )
            timeout = timeout or self.__config.get(
                'POLLING_TIMEOUT', DEFAULT_GET_BROWSER_TIMEOUT,
            )
            args = {
                True: (self.remote, self.__browser_name),
                False: (get_local_browser, self.__browser_name),
            }

            self.__browser = waiting_for(
                get_browser,
                delay=delay,
                timeout=timeout,
                exc_cls=SeleniumExError,
                args=args[bool(self.__config.get('USE_REMOTE', False))],
                message='Browser "{}" has not been started for "{}" sec.'.format(
                    self.__browser_name, timeout,
                ),
            )

            self.__is_running = True

    def stop(self):
        if self.__is_running:
            with self.__browser.disable_polling():
                self.__browser.close()
                self.__browser.quit()
            self.__is_running = False

    def remote(self, driver_name):
        driver_name = driver_name.lower()
        remote_config = self.__config.get('REMOTE')

        if not remote_config:
            raise SeleniumExError(
                'settings for remote is not found in config',
            )

        logger.debug(
            'Remote config: {}'.format(str(remote_config)),
        )

        if not remote_config.get('desired_capabilities'):
            capabilities = get_capabilities(driver_name)
            capabilities.update(
                remote_config.pop('capabilities', {}).get(driver_name, {}),
            )
            remote_config['desired_capabilities'] = capabilities

        driver = drivers.RemoteWebDriver(**remote_config)
        return browser.create(self, driver)

    def ie(self):
        ie_config = self.__config.get('IE')

        if not ie_config:
            raise SeleniumExError(
                'settings for ie browser is not found in config',
            )

        logger.debug(
            'Ie config: {}'.format(str(ie_config)),
        )

        driver = drivers.IeWebDriver(**ie_config)
        return browser.create(self, driver)

    def chrome(self):
        """
        :rtype: selenium.webdriver.chrome.webdriver.WebDriver
        """
        chrome_config = self.__config.get('CHROME')

        if not chrome_config:
            raise SeleniumExError(
                'settings for chrome browser is not found in config',
            )

        logger.debug(
            'Chrome config: {}'.format(str(chrome_config)),
        )

        driver = drivers.ChromeWebDriver(**chrome_config)
        return browser.create(self, driver)

    def firefox(self):
        firefox_config = self.__config.get('FIREFOX', {})

        logger.debug(
            'Firefox config: {}'.format(str(firefox_config)),
        )

        driver = drivers.FirefoxWebDriver(**firefox_config)
        return browser.create(self, driver)

    def phantomjs(self):
        phantom_config = self.__config.get('PHANTOMJS')

        if not phantom_config:
            raise SeleniumExError(
                'settings for phantom js is not found in config',
            )

        logger.debug(
            'PhantomJS config: {}'.format(str(phantom_config)),
        )

        driver = drivers.PhantomJSWebDriver(**phantom_config)
        return browser.create(self, driver)

    def opera(self):
        opera_config = self.__config.get('OPERA')

        if not opera_config:
            raise SeleniumExError(
                'settings for opera browser is not found in config',
            )

        logger.debug(
            'Opera config: {}'.format(str(opera_config)),
        )

        driver = drivers.OperaWebDriver(**opera_config)
        return browser.create(self, driver)



# -*- coding: utf-8 -*-

"""
Multiprocessing utils
"""


class MPSupportedValue(object):

    def __init__(self, value=None):
        self._value = value

    @property
    def value(self):
        val = getattr(self._value, 'value', None)
        if val is not None:
            return self._value.value

        return self._value

    @value.setter
    def value(self, value):
        if hasattr(self._value, 'value'):
            self._value.value = value
        else:
            self._value = value

    def set(self, value):
        self._value = value



# -*- coding: utf-8 -*-

from ...exceptions import SeismographError


class AlchemyError(SeismographError):
    pass


class ConfigurationError(AlchemyError):
    pass


class NotFound(AlchemyError):
    pass


class InvalidBindKey(AlchemyError):
    pass



# -*- coding: utf-8 -*-

import time

from ..exceptions import TimeoutException


def waiting_for(func, timeout=None, exc_cls=None, message=None, delay=None, args=None, kwargs=None):
    args = args or tuple()
    kwargs = kwargs or dict()

    timeout = timeout or 0
    message = message or 'Timeout "{}" exceeded'.format(timeout)

    if timeout:
        t_start = time.time()

        while time.time() <= t_start + timeout:
            result = func(*args, **kwargs)

            if result:
                return result

            if delay:
                time.sleep(delay)
        else:
            if exc_cls:
                raise exc_cls(message)
            raise TimeoutException(message)

    result = func(*args, **kwargs)

    if result:
        return result
    if exc_cls:
        raise exc_cls(message)
    raise TimeoutException(message)


def call_to_chain(chain, method_name, *args, **kwargs):
    for obj in chain:
        if method_name:
            getattr(obj, method_name)(*args, **kwargs)
        else:
            obj(*args, **kwargs)


def measure_time():
    start_time = time.time()
    return lambda: time.time() - start_time



# -*- coding: utf-8 -*-

from contextlib import contextmanager

from six import with_metaclass

from ...utils import pyv
from .query import QueryObject


def key(page_element):
    return page_element.__key__()


def query_set(page_element):
    return page_element.__query_set__()


class ProxyObject(object):

    def __init__(self, we):
        self.__we = we

    def __getattr__(self, item):
        return getattr(self.__we, item)

    def __repr__(self):
        return repr(self.__we)

    def __str__(self):
        return str(self.__we)

    def __dir__(self):
        return dir(self.__we)


class PageCache(dict):

    def __init__(self, *args, **kwargs):
        super(PageCache, self).__init__(*args, **kwargs)

        self.is_eternal = False

    @contextmanager
    def restore(self):
        current_data = self.copy()
        try:
            yield
        finally:
            self.update(current_data)

    def clear(self):
        if not self.is_eternal:
            super(PageCache, self).clear()


class PageElement(object):

    def __init__(self, *args, **options):
        assert bool(args), 'Element can not be created without arguments'

        self.__class = None
        self.__query_set = []

        self.__cached = options.get('cached', None)

        for a in args:
            if pyv.is_class_type(a):
                self.__class = a
                if self.__cached is not False:
                    self.__cached = True
                break
            elif isinstance(a, PageElement):
                self.__query_set.extend(query_set(a))
            elif isinstance(a, QueryObject):
                self.__query_set.append(a)
            else:
                raise ValueError(
                    'Incorrect page object argument {}'.format(a),
                )

        self.__key = options.get('key', None)
        self.__call = options.get('call', None)
        self.__index = options.get('index', None)
        self.__is_list = options.get('is_list', False)
        self.__property = options.get('property', None)
        self.__we_class = options.get('we_class', None)
        self.__list_class = options.get('list_class', None)
        self.__wait_timeout = options.get('wait_timeout', None)

        if self.__list_class and not self.__is_list:
            raise ValueError(
                '"list_class" can not be using without "is_list"',
            )

        if self.__call:
            if self.__list_class:
                self.__list_class = type(
                    self.__list_class.__name__,
                    (self.__list_class, ),
                    {'__call__': self.__call},
                )
            elif self.__we_class and not self.__is_list:
                self.__we_class = type(
                    self.__we_class.__name__,
                    (self.__we_class, ),
                    {'__call__': self.__call},
                )
            else:
                if self.__is_list:
                    self.__list_class = type(
                        'CallableObject',
                        (ProxyObject, ),
                        {'__call__': self.__call},
                    )
                else:
                    self.__we_class = type(
                        'CallableObject',
                        (ProxyObject, ),
                        {'__call__': self.__call},
                    )

    def __getattr__(self, item):
        # for IDE only
        raise AttributeError(item)

    def __make_object__(self, page):
        if self.__cached and id(self) in page.cache:
            result = page.cache[id(self)]
        else:
            if self.__class:
                result = self.__class(page.area)
            else:
                we = None

                for query_object in self.__query_set[:-1]:
                    if we:
                        we = query_object(we).first()
                    else:
                        we = query_object(page.area).first()

                query_object = self.__query_set[-1:][0]

                if we:
                    query_result = query_object(we)
                else:
                    query_result = query_object(page.area)

                if self.__wait_timeout:
                    query_result.wait(
                        self.__wait_timeout,
                    )

                if self.__is_list:
                    result = query_result.all()
                elif self.__index:
                    result = query_result.get(
                        self.__index,
                    )
                else:
                    result = query_result.first()

        if self.__cached:
            page.cache[id(self)] = result

        return result

    def __get__(self, instance, owner):
        if instance is None:
            return self

        obj = self.__make_object__(instance)

        if self.__we_class:
            if self.__is_list:
                for o in obj:
                    index = obj.index(o)
                    obj[index] = self.__we_class(o)
            else:
                obj = self.__we_class(obj)

        if self.__list_class:
            obj = self.__list_class(obj)

        if self.__property:
            return self.__property(obj)

        return obj

    def __set__(self, instance, value):
        raise TypeError('\'PageObject\' is not settable')

    def __key__(self):
        return self.__key

    def __query_set__(self):
        return self.__query_set


class PageApi(object):

    def __init__(self, page):
        self.__page = page

    @property
    def page(self):
        return self.__page

    @property
    def browser(self):
        return self.__page.browser


class PageMeta(type):
    """
    Factory for creating page object class
    """

    def __new__(mcs, name, bases, dct):
        cls = type.__new__(mcs, name, bases, dct)

        setattr(cls, '__dct__', {})

        page_objects = (
            (a, getattr(cls, a, None))
            for a in dir(cls)
            if not a.startswith('_')
            and
            isinstance(getattr(cls, a, None), PageElement)
        )

        for atr_name, page_object in page_objects:
            if key(page_object):
                cls.__dct__[key(page_object)] = atr_name

        return cls


class _Base(with_metaclass(PageMeta, object)):

    def __init__(self, proxy=None):
        self.__proxy = proxy
        self.__cache = PageCache()

    def __getitem__(self, item):
        try:
            return getattr(self, self.__class__.__dct__[item])
        except (KeyError, AttributeError):
            raise KeyError(item)

    def __getattr__(self, item):
        return getattr(self.area, item)

    @property
    def _proxy(self):
        return self.__proxy

    @property
    def area(self):
        raise NotImplementedError(
            'Property "area" is not implemented in "{}"'.format(
                self.__class__.__name__,
            ),
        )

    @property
    def browser(self):
        if self.__proxy:
            return self.__proxy.browser
        return None

    @property
    def cache(self):
        return self.__cache

    def bind_to(self, proxy):
        self.__proxy = proxy


class Page(_Base):

    __area__ = None
    __url_path__ = None
    __api_class__ = None

    def __init__(self, *args, **kwargs):
        super(Page, self).__init__(*args, **kwargs)

        if self.__api_class__:
            self.__api = self.__api_class__(self)
        else:
            self.__api = None

    @property
    def api(self):
        return self.__api

    @property
    def area(self):
        if self.__area__:
            if not isinstance(self.__area__, QueryObject):
                raise TypeError(
                    '"__area__" can be instance of QueryObject only',
                )
            return self.__area__(self._proxy).first()

        return self._proxy

    def open(self, **kwargs):
        if self.__url_path__:
            self.cache.clear()
            self.browser.router.go_to(
                self.__url_path__.format(**kwargs),
            )
        else:
            raise RuntimeError(
                'You should to set "__url_path__" attribute value for usage "show" method',
            )

    def refresh(self):
        self.cache.clear()
        self._proxy.browser.refresh()


class PageItem(_Base):

    __area__ = None
    __nested__ = True

    @property
    def we(self):
        if self._proxy and self._proxy.is_web_element:
            return self._proxy
        return None

    @property
    def area(self):
        proxy = self._proxy if self.__nested__ else self._proxy.browser

        if self.__area__:
            if not isinstance(self.__area__, QueryObject):
                raise TypeError(
                    '"__area__" can be instance of QueryObject only',
                )
            return self.__area__(proxy).first()

        return proxy



# -*- coding: utf-8 -*-

from optparse import OptionParser


DEFAULT_PORT = 5000
DEFAULT_HOST = '127.0.0.1'


def get_parser():
    parser = OptionParser('python -m seismograph.ext.mock_server [options]')

    parser.add_option(
        '-p', '--port',
        type=int,
        dest='PORT',
        default=DEFAULT_PORT,
        help='Server port'
    )
    parser.add_option(
        '-i', '--host',
        dest='HOST',
        default=DEFAULT_HOST,
        help='Server host'
    )
    parser.add_option(
        '-m', '--mocks-dir',
        dest='MOCKS_DIR',
        default=None,
        help='Path to dir within mock files'
    )
    parser.add_option(
        '-t', '--type',
        dest='SERVER_TYPE',
        default='json_api',
        help='Server type. Can be in ({}). "json_api" by default'.format(
            ', '.join(('"simple"', '"json_api"')),
        )
    )

    parser.add_option(
        '--no-debug',
        dest='NO_DEBUG',
        action='store_false',
        default=True,
        help='No use debug for output',
    )
    parser.add_option(
        '--multiprocessing',
        dest='MULTIPROCESSING',
        action='store_true',
        default=False,
        help='Use fork server',
    )
    parser.add_option(
        '--threading',
        dest='THREADING',
        action='store_true',
        default=False,
        help='Use thread server',
    )
    parser.add_option(
        '--gevent',
        dest='GEVENT',
        action='store_true',
        default=False,
        help='Use gevent wsgi server',
    )

    return parser


def main():
    parser = get_parser()
    options, _ = parser.parse_args()

    if options.GEVENT:
        from seismograph.utils import pyv
        pyv.check_gevent_supported()

        from gevent.monkey import patch_all
        patch_all(thread=False)

    from seismograph.ext.mocker import SERVER_TYPES

    try:
        mock_server_class = SERVER_TYPES[options.SERVER_TYPE]
    except KeyError:
        raise ValueError('Incorrect server type')

    server = mock_server_class(
        options.MOCKS_DIR,
        host=options.HOST,
        port=options.PORT,
        debug=options.NO_DEBUG,
        gevent=options.GEVENT,
        threading=options.THREADING,
        multiprocessing=options.MULTIPROCESSING,
    )
    server.serve_forever()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from functools import wraps


RESET = '\033[0m'


def reset_color(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        return '{}{}'.format(
            f(*args, **kwargs), RESET,
        )
    return wrapper


@reset_color
def green(string):
    return '\033[32m{}\033[42m'.format(string)


@reset_color
def red(string):
    return '\033[31m{}\033[41m'.format(string)


@reset_color
def yellow(string):
    return '\033[33m{}\033[43m'.format(string)


@reset_color
def blue(string):
    return '\033[34m{}\033[44m'.format(string)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from gevent import Greenlet
from gevent.wsgi import WSGIServer


class RunServerInGreenlet(Greenlet):

    def __init__(self, mock_server):
        self._httpd = None
        self._mock_server = mock_server

        super(RunServerInGreenlet, self).__init__()

    def run(self):
        self._httpd = WSGIServer(
            (self._mock_server.host, self._mock_server.port),
            self._mock_server,
            log='default' if self._mock_server.debug else None,
        )
        self._httpd.start()

    def stop(self):
        if self._httpd:
            self._httpd.stop()
